/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Flem
 *
 * The Initial Developer of the Original Code is  BASTIAN Mathieu <mathieu.bastian@gmail.com>.
 * Portions created by the Initial Developer are Copyright (C) 2006-2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Mathieu Jacomy
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
 
//-------------------------------------------------------------------------------------------------------
//--------------------------------------         Utils         ---------------------------------------------
//-------------------------------------------------------------------------------------------------------

function LOG(msg) {
  var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
  consoleService.logStringMessage(msg);
}

function flem_EnableButton(btn)
{
    var button = document.getElementById(btn);
	button.setAttribute("disabled", false);
}

function flem_DisableButton(btn)
{
    var button = document.getElementById(btn);
	button.setAttribute("disabled", true);
}

function flem_IsEnable(btn)
{
    var button = document.getElementById(btn);
	var res = !button.disabled;
	return res;
}


//------------------------------------------------------------------------------------------------------
//--------------------------------------         Init         ---------------------------------------------
//------------------------------------------------------------------------------------------------------

function initOnceWebAtlasUrlLoader() 
{
	window.removeEventListener("load", initOnceWebAtlasUrlLoader, true);
	flem_urlLoader.init();
}

function flemStr()
{
	return document.getElementById("flem-string-bundle");
}

window.addEventListener("load", initOnceWebAtlasUrlLoader, true);

var gPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);	

//--------------------------------------------------------------------------------------------------------------
//--------------------------------------         Url Loader         ---------------------------------------------
//--------------------------------------------------------------------------------------------------------------

var flem_urlLoader = {
	filePicker : 0,
	filePickerSave : 0,
	
	init: function()
	{
		if(document.getElementById("navigator-toolbox") == null)
			return;
	
		LOG("Flem loaded");
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		this.filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		this.filePicker.init(window, flemStr().getString("flem_selectFile"), nsIFilePicker.modeOpen);
		this.filePicker.appendFilter(flemStr().getString("flem_dataFile")+" (*.csv;*.txt)","*.csv; *.txt");
		
		this.filePickerSave = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		this.filePickerSave.init(window, flemStr().getString("flem_saveFile"),nsIFilePicker.modeSave);
		this.filePickerSave.appendFilter(flemStr().getString("flem_dataFile")+" (*.csv;*.txt)","*.csv; *.txt");
		
		flem_DisableButton("urlloader-toolbar-prec");
		flem_DisableButton("urlloader-toolbar-next");
		flem_DisableButton("urlloader-toolbar-reset");
		flem_DisableButton("urlloader-toolbar-deldoublons");
		flem_DisableButton("urlloader-toolbar-exportUnvisited");
		flem_DisableButton("urlloader-toolbar-stopslideshow");
		flem_DisableButton("urlloader-toolbar-startslideshow");
	},
	
	openFile: function()
	{
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var res = this.filePicker.show();
		if (res == nsIFilePicker.returnOK)
		{
			var thefile = this.filePicker.file;
			var fileString = this.readFile(thefile);
			this.getUrl(fileString);
			flem_urlView.start();
		}
	
	},
	
	pasteFromClipboard: function()
	{
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip) return false;

		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans) return false;
		trans.addDataFlavor("text/unicode");
		
		clip.getData(trans,clip.kGlobalClipboard);

		var str = new Object();
		var strLength = new Object();
		var pastetext;

		trans.getTransferData("text/unicode",str,strLength);
		if (str) str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
		if (str) pastetext = str.data.substring(0,strLength.value / 2);
		this.getUrl(pastetext);
		flem_urlView.start();
	},
	
	exportUnvisited:function()
	{
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var res = this.filePickerSave.show();
		if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace )
		{
			var thefile = this.filePickerSave.file;
			var text = this.getUnvisited();
			this.saveFile(thefile,text);
		}
	},
	
	readFile: function(file)
	{
		
		try {
			netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
		} catch (e) {
			alert("Permission to read file was denied.");
		}

		
		//Mode de lecture du fichier, un flux est ncessaire
		//Le second argument dfinit les diffrents modes de lecture parmis
		//PR_RDONLY     =0x01 lecture seulement
		//PR_WRONLY     =0x02 criture seulement
		//PR_RDWR       =0x04 lecture ou criture
		//PR_CREATE_FILE=0x08 si le fichier n'existe pas, il est cr (sinon, sans effet)
		//PR_APPEND     =0x10 le fichier est positionn  la fin avant chaque criture
		//PR_TRUNCATE   =0x20 si le fichier existe, sa taille est rduite  zro
		 //PR_SYNC       =0x40 chaque criture attend que les donnes ou l'tat du fichier soit mis  jour
		//PR_EXCL       =0x80 idem que PR_CREATE_FILE, sauf que si le fichier existe, NULL est retourne
		//Le troisime argument dfinit les droits
		var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
	         .createInstance( Components.interfaces.nsIFileInputStream );
		is.init(file, 0x01, 00004, null);

		//Lecture du fichier en mode binaire
		var sis = Components.classes["@mozilla.org/binaryinputstream;1"]
	          .createInstance(Components.interfaces.nsIBinaryInputStream);
		sis.setInputStream( is );
		var output = sis.readBytes(sis.available());

		return output;

	},
	
	saveFile: function(file, content)
	{
		try {
			netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
		} catch (e) {
			alert("Permission to save file was denied.");
		}

		if ( file.exists() == false ) {
			file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
		}

		var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
			.createInstance( Components.interfaces.nsIFileOutputStream );
			
		/* Open flags 
		#define PR_RDONLY       0x01
		#define PR_WRONLY       0x02
		#define PR_RDWR         0x04
		#define PR_CREATE_FILE  0x08
		#define PR_APPEND      0x10
		#define PR_TRUNCATE     0x20
		#define PR_SYNC         0x40
		#define PR_EXCL         0x80
		*/
		/*
		** File modes ....
		**
		** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
		** The 'mode' argument may be ignored by PR_Open on other platforms.
		**
		**   00400   Read by owner.
		**   00200   Write by owner.
		**   00100   Execute (search if a directory) by owner.
		**   00040   Read by group.
		**   00020   Write by group.
		**   00010   Execute by group.
		**   00004   Read by others.
		**   00002   Write by others
		**   00001   Execute by others.
		**
		*/
		
		outputStream.init( file, 0x02 | 0x08 | 0x20, 420, 0 );
		var written = outputStream.write( content, content.length );
		if(written!=content.length)
			alert("Erreur d'criture");
		outputStream.close();
	},
	
	getUrl: function(fileString)
	{
		var lignes = fileString.split("\n");
		var ligne,cellules,cellule,cellulesB,celluleC, url,urlObject,nocolumn;
		var lineArray = new Array();
		var validLine = false;
		for(var i=1; i < lignes.length; i++)
		{
			ligne = lignes[i-1];
			cellules = ligne.split(";");
			cellulesB = ligne.split(",");
			cellulesC = ligne.split(" ");
			if(cellulesB.length > cellules.length)
			{
				if(cellulesC.length > cellulesB.length)
					cellules = cellulesC;
				else
					cellules = cellulesB;
			}
			validLine = false;
			for(var j=0; j < cellules.length; j++)
			{
				cellule = cellules[j];
				if(cellule != "" && cellule !="\r") {
					var http = cellule.indexOf("http://");
					var https = cellule.indexOf("https://");
					var www = cellule.indexOf("www.");
					if(!validLine && (http!=-1 || https!=-1 || www!=-1))
					{
						if(http!=-1) 
						{
							url = cellule.substring(http, cellule.length);
						} 
						else if(https!=-1) 
						{
							url = cellule.substring(https, cellule.length);
						} 
						else if(www!= -1) 
						{
							url = cellule.substring(www, cellule.length);
						}
					
						//Valide url
						validLine = true;
						
						//Where is the url column, useful for export
						nocolumn = j;
						
					}
					else
					{
						cellule = cellule.replace("\r","");
						cellule = cellule.replace("\n","");
						cellule = cellule.replace("\r\n","");
						lineArray.push(cellule);
					}
				}
			}
			
			//Add the url line
			if(validLine)
			{
				urlObject = new FlemUrl(flem_urlView.urlId, url, lineArray,nocolumn);
				flem_urlView.push(urlObject);
			}
			
			lineArray = new Array();
		}
	},
	
	reset: function()
	{
		flem_urlView.pointer=-1;
		flem_urlView.urlId=0;
		flem_urlView.list = new Array();
		flem_urlView.indexedList = new Array();
		flem_urlView.item = 0;
		
		var tooltip = document.getElementById("commtip");
		//Clear tooltip
		while(tooltip.hasChildNodes())
		{
			tooltip.removeChild(tooltip.firstChild);
		}
		
		
		var combo = document.getElementById("urlloader-combo");
		//Clear combo
		while(combo.hasChildNodes())
		{
			combo.removeChild(combo.firstChild);
		}
		document.getElementById("urlloader-combobox").setAttribute("class", "");
		document.getElementById("urlloader-combobox").setAttribute("label", "");
		
		flem_DisableButton("urlloader-toolbar-next");
		flem_DisableButton("urlloader-toolbar-prec");
		flem_DisableButton("urlloader-toolbar-reset");
		flem_DisableButton("urlloader-toolbar-deldoublons");
		flem_DisableButton("urlloader-toolbar-startslideshow");
		flem_DisableButton("urlloader-toolbar-stopslideshow");
		
	},
	
	
	delDouble: function()
	{
		var newTab = new Array();
		var newIndexedTab = new Array();
		var q=0;
		var u;
		for(var x=0;x<flem_urlView.list.length;x++)
		{
			u = flem_urlView.list[x];
			if(u.doublon==false)
			{
				u.id = q;
				newTab.push(u);
				newIndexedTab[u.url] = 1;
				q++;
			}
		}
		flem_urlView.list = newTab;
		flem_urlView.indexedList = newIndexedTab;
		flem_urlView.urlId = newTab.length;
		flem_urlView.start();
	},
	
	getUnvisited: function()
	{
		var texte = "";
		var i,j;
		for(i=0;i<flem_urlView.list.length;i++)
		{
			var urlObj = flem_urlView.list[i];
			if(urlObj.visited==false)
			{
				//Before url column
				for(j=0;j<urlObj.nocolumn;j++)
				{
					texte+=urlObj.commentaires[j]+";";
				}
				
				//Url column
				texte+=urlObj.url;
				
				//After
				for(j=urlObj.nocolumn;j<urlObj.commentaires.length;j++)
				{
					texte+=";"+urlObj.commentaires[j];
				}
				texte+="\r";
			}
		}
		return texte;
	},
		
};

//-------------------------------------------------------------------------------------------------------
//-------------------------------------         Objects         --------------------------------------------
//-------------------------------------------------------------------------------------------------------

function FlemUrl(_id,_url,_commentaires,_nocolumn) 
{
	this.url = ""+_url;
	this.id = _id;
	this.commentaires = _commentaires;
	this.visited=false;
	this.doublon=false;
	this.nocolumn = _nocolumn;
}


//-------------------------------------------------------------------------------------------------------
//----------------------------------------         UI         -----------------------------------------------
//-------------------------------------------------------------------------------------------------------

var flem_urlView = {
	list : new Array(),
	indexedList : new Array(),
	urlId : 0,
	pointer: -1,
	item: 0,
	
	start: function()
	{
		if(this.list.length > 0)
		{
			flem_EnableButton("urlloader-toolbar-reset");
			flem_EnableButton("urlloader-toolbar-deldoublons");
			flem_EnableButton("urlloader-toolbar-exportUnvisited");
			flem_EnableButton("urlloader-toolbar-startslideshow");
			
			if(this.list.length > 1)
				flem_EnableButton("urlloader-toolbar-next");
		
			this.fillCombo();
			document.getElementById("urlloader-combobox").setAttribute("label", "--- "+this.list.length+" "+flemStr().getString("flem_urlLoaded"));
			document.getElementById("urlloader-combobox").setAttribute("class" , "combotitle");
		}
	
	},
	
	fillCombo: function()
	{
		var combo = document.getElementById("urlloader-combo");
		
		//Clear combo
		while(combo.hasChildNodes())
		{
			combo.removeChild(combo.firstChild);
		}
		
		var item;
		for(var i=0;i<this.list.length;i++)
		{
			item = document.createElement("menuitem");
			item.setAttribute("label", this.list[i].id+" : "+this.list[i].url);
			item.setAttribute("value", this.list[i].id);
			if(this.list[i].visited==true)
				item.setAttribute("class", "txtgreen");
			
			combo.appendChild(item);
		}
	},

	push: function (u) 
	{
		
		//Check if doublon
		if(this.indexedList[u.url] != null)
		{
			u.doublon=true;
		}
		else
		{
			//Store to indexed tab to further double test
			this.indexedList[u.url] = 1;
		}
		
		this.list.push(u);
		this.urlId++;
	},
	
	getNext: function(event, forceNewTab)
	{
		if(this.item != 0)
		{
			this.item.setAttribute("class", "txtgreen");
		}
		else
		{
			//Disable blue background
			document.getElementById("urlloader-combobox").setAttribute("class" , "");
		}
		
		this.pointer++;
		
		if(this.pointer > 0)
			flem_EnableButton("urlloader-toolbar-prec");
		
		if(this.pointer+1 == this.list.length)
		{
			//Block next
			flem_DisableButton("urlloader-toolbar-next");
		}

		var combo = document.getElementById("urlloader-combo");
		this.item = combo.childNodes[this.pointer];
		this.item.setAttribute("class", "txtcurrent");
		document.getElementById("urlloader-combobox").selectedIndex = this.pointer;

		var openNewTab = false;
		if(forceNewTab || event.ctrlKey)
		{
			openNewTab = true;
		}
		
		this.showUrl(this.list[this.pointer],openNewTab);

	},
	
	getPrec: function(event, forceNewTab)
	{
		this.pointer--;
		this.item.setAttribute("class", "txtgreen");
		
		if(this.pointer == 0)
		{
			//Block prec
			flem_DisableButton("urlloader-toolbar-prec");
		}
		
		var combo = document.getElementById("urlloader-combo");
		this.item = combo.childNodes[this.pointer];
		this.item.setAttribute("class", "txtcurrent");
		document.getElementById("urlloader-combobox").selectedIndex = this.pointer;
		
		flem_EnableButton("urlloader-toolbar-next");
		
		
		
		var openNewTab = false;
		if(forceNewTab || event.ctrlKey)
		{
			openNewTab = true;
		}
		
		this.showUrl(this.list[this.pointer],openNewTab);
	
	},
	
	selectUrl: function(event)
	{
		if(this.item == 0)
		{
			//Disable blue background
			document.getElementById("urlloader-combobox").setAttribute("class" , "");
		} else
		{
			this.item.setAttribute("class", "txtgreen");
		}
		
		var combo = document.getElementById("urlloader-combo");
		var index = document.getElementById("urlloader-combobox").selectedIndex;
		this.item = combo.childNodes[index];
		this.item.setAttribute("class", "txtcurrent");
		this.pointer = index;
		
		if(this.pointer ==0)
			flem_DisableButton("urlloader-toolbar-prec");
		else
			flem_EnableButton("urlloader-toolbar-prec");
			
		if(this.pointer+1 == this.list.length)
			flem_DisableButton("urlloader-toolbar-next");
		else
			flem_EnableButton("urlloader-toolbar-next");
			
		var openNewTab = false;
		if(event.ctrlKey)
		{
			openNewTab = true;
		}
			
		this.showUrl(this.list[this.pointer],openNewTab);
	},
	
	
	showUrl:function(url, inNewTab)
	{
		if(url!=null)
		{
			var urlStr = url.url;
			url.visited = true;
			if(urlStr.substring(0,4) != "http")
				urlStr= "http://"+urlStr;
			
			if(inNewTab==true)
			{
				gBrowser.selectedTab = gBrowser.addTab(urlStr);
			}
			else
			{
				getBrowser().selectedBrowser.setAttribute("src",urlStr);
			}	
		}
	},
	
	fillTooltip:function()
	{
		var tooltip = document.getElementById("commtip");
	
		//Clear tooltip
		while(tooltip.hasChildNodes())
		{
			tooltip.removeChild(tooltip.firstChild);
		}
	
		//Fill
		var vbox = document.createElement("vbox");
		var lbl, comm;
		for(var i=0;i<this.list[this.pointer].commentaires.length;i++)
		{
			comm = this.list[this.pointer].commentaires[i];
			lbl = document.createElement("label");
			lbl.setAttribute("value",  comm);
			vbox.appendChild(lbl);
			
			if(i!=this.list[this.pointer].commentaires.length -1)
			{
				//Rajout d'un espace sauf dernier
				lbl = document.createElement("label");
				lbl.setAttribute("value",  "  ");
				vbox.appendChild(lbl);
			}
		}
		tooltip.appendChild(vbox);
		
		return true;
	},
	
};

//-------------------------------------------------------------------------------------------------------
//------------------------------------         Slideshow         -------------------------------------------
//-------------------------------------------------------------------------------------------------------

var flem_urlSlideshow = {
	filePickerSave : 0,
	timer : 0,
	
	startSlideshow:function()
	{
		document.getElementById("urlloader-toolbar-startslideshow").setAttribute("disabled", true);
		document.getElementById("urlloader-toolbar-stopslideshow").setAttribute("disabled", false);
		this.timer = setInterval("flem_urlSlideshow.step()", gPref.getIntPref("extensions.flem.slideshow.interval")*1000);
	},
	
	step: function()
	{
		var openinewtab;
		if(flem_IsEnable("urlloader-toolbar-next"))
		{
			if(gPref.getBoolPref("extensions.flem.slideshow.openinnewtab"))
			{
				openinewtab=1;
			}
			else
			{
				openinewtab=0;
			}
			
			flem_urlView.getNext(0, openinewtab)
		}
		else
		{
			if(gPref.getBoolPref("extensions.flem.slideshow.loop"))
			{
				if(flem_urlView.list.length > 0)
				{
					flem_urlView.pointer=-1;
					flem_urlView.item = 0;
					flem_EnableButton("urlloader-toolbar-next");
				}
				else
				{
					this.stopSlideshow();
					return;
				}
				flem_urlView.getNext(0, openinewtab)
			}
			else
			{
				this.stopSlideshow();
			}
		}
	
	},
	
	stopSlideshow:function()
	{
		document.getElementById("urlloader-toolbar-startslideshow").setAttribute("disabled", false);
		document.getElementById("urlloader-toolbar-stopslideshow").setAttribute("disabled", true);
		clearInterval ( this.timer )
	},
	
	configureSlideshow:function()
	{
		var params;
		var dialog = window.openDialog("chrome://webatlasurlloader/content/configureSlideshow.xul", "","chrome, modal, centerscreen", params);		
		
	},
	
	loadDialogData:function(win)
	{
		var interval = gPref.getIntPref("extensions.flem.slideshow.interval");
		var loop = gPref.getBoolPref("extensions.flem.slideshow.loop");
		var openinewtab = gPref.getBoolPref("extensions.flem.slideshow.openinnewtab");
		
		win.document.getElementById("flem-configureslideshow-dialog-interval").setAttribute("value", interval);
		win.document.getElementById("flem-configureslideshow-dialog-loopcheckbox").setAttribute("checked", loop);
		win.document.getElementById("flem-configureslideshow-dialog-innewtab").setAttribute("checked", openinewtab);
	},
	
	saveDialogData:function(win)
	{
		var interval = win.document.getElementById("flem-configureslideshow-dialog-interval").value;
		var loop = win.document.getElementById("flem-configureslideshow-dialog-loopcheckbox").checked;
		var openinewtab = win.document.getElementById("flem-configureslideshow-dialog-innewtab").checked;
		
		try {
			if(isNaN(interval)) { throw "The interval must be a number" }
			gPref.setIntPref("extensions.flem.slideshow.interval", interval);
			gPref.setBoolPref("extensions.flem.slideshow.loop", loop);
			gPref.setBoolPref("extensions.flem.slideshow.openinnewtab", openinewtab);
		} catch(e) { alert(""+e); return false; }
		return true;
	},
	
};